@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import argparse
 
 from pdm import termui
@@ -5,6 +7,7 @@ from pdm.cli import actions
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.hooks import HookManager
 from pdm.cli.options import skip_option
+from pdm.models.backends import _BACKENDS, BuildBackend, get_backend
 from pdm.models.python import PythonInfo
 from pdm.project import Project
 from pdm.utils import get_user_email_from_git, get_venv_like_prefix
@@ -33,29 +36,42 @@ class Command(BaseCommand):
             action="store_true",
             help="Don't ask questions but use default values",
         )
+        parser.add_argument("--python", help="Specify the Python version/path to use")
+        parser.add_argument(
+            "--backend", choices=list(_BACKENDS), help="Specify the build backend"
+        )
         parser.set_defaults(search_parent=False)
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
         from pdm.cli.commands.venv.utils import get_venv_python
 
         hooks = HookManager(project, options.skip)
-        if project.pyproject_file.exists():
+        if project.pyproject.exists:
             project.core.ui.echo(
-                "pyproject.toml already exists, update it now.", style="cyan"
+                "pyproject.toml already exists, update it now.", style="primary"
             )
         else:
-            project.core.ui.echo("Creating a pyproject.toml for PDM...", style="cyan")
+            project.core.ui.echo(
+                "Creating a pyproject.toml for PDM...", style="primary"
+            )
         self.set_interactive(not options.non_interactive)
 
         if self.interactive:
-            python = actions.do_use(project, ignore_requires_python=True, hooks=hooks)
+            python = actions.do_use(
+                project,
+                options.python or "",
+                first=bool(options.python),
+                ignore_remembered=True,
+                ignore_requires_python=True,
+                hooks=hooks,
+            )
             if (
                 project.config["python.use_venv"]
                 and get_venv_like_prefix(python.executable) is None
             ):
                 if termui.confirm(
                     "Would you like to create a virtualenv with "
-                    f"[green]{python.executable}[/]?",
+                    f"[success]{python.executable}[/]?",
                     default=True,
                 ):
                     try:
@@ -67,28 +83,56 @@ class Command(BaseCommand):
                         project.core.ui.echo(
                             f"Error occurred when creating virtualenv: {e}\n"
                             "Please fix it and create later.",
-                            style="red",
+                            style="error",
                             err=True,
                         )
         else:
             python = actions.do_use(
-                project, "3", True, ignore_requires_python=True, save=False, hooks=hooks
+                project,
+                options.python or "3",
+                first=True,
+                ignore_remembered=True,
+                ignore_requires_python=True,
+                save=False,
+                hooks=hooks,
             )
         if get_venv_like_prefix(python.executable) is None:
             project.core.ui.echo(
                 "You are using the PEP 582 mode, no virtualenv is created.\n"
                 "For more info, please visit https://peps.python.org/pep-0582/",
-                style="green",
+                style="success",
             )
         is_library = (
-            termui.confirm("Is the project a library that will be uploaded to PyPI")
+            termui.confirm(
+                "Is the project a library that is installable?\n"
+                "A few more questions will be asked to include a project name "
+                "and build backend"
+            )
             if self.interactive
             else False
         )
+        build_backend: type[BuildBackend] | None = None
         if is_library:
             name = self.ask("Project name", project.root.name)
             version = self.ask("Project version", "0.1.0")
             description = self.ask("Project description", "")
+            if options.backend:
+                build_backend = get_backend(options.backend)
+            elif self.interactive:
+                all_backends = list(_BACKENDS)
+                project.core.ui.echo("Which build backend to use?")
+                for i, backend in enumerate(all_backends):
+                    project.core.ui.echo(f"{i}. [success]{backend}[/]")
+                selected_backend = termui.ask(
+                    "Please select",
+                    prompt_type=int,
+                    choices=[str(i) for i in range(len(all_backends))],
+                    show_choices=False,
+                    default=all_backends.index("pdm-pep517"),
+                )
+                build_backend = get_backend(all_backends[int(selected_backend)])
+            else:
+                build_backend = get_backend("pdm-pep517")
         else:
             name, version, description = "", "", ""
         license = self.ask("License(SPDX name)", "MIT")
@@ -110,6 +154,7 @@ class Command(BaseCommand):
             author=author,
             email=email,
             python_requires=python_requires,
+            build_backend=build_backend,
             hooks=hooks,
         )
         if self.interactive:
