@@ -1,16 +1,18 @@
+from __future__ import annotations
+
 import abc
 import shutil
 import subprocess
 import sys
 from pathlib import Path
-from typing import Any, List, Mapping, Optional, Tuple, Type
+from typing import Any, Iterable, Mapping
 
-from pdm.project import Project
 from pdm import termui
 from pdm.cli.commands.venv.utils import get_venv_prefix
+from pdm.compat import cached_property
 from pdm.exceptions import PdmUsageError, ProjectError
 from pdm.models.python import PythonInfo
-from pdm.utils import cached_property
+from pdm.project import Project
 
 
 class VirtualenvCreateError(ProjectError):
@@ -20,10 +22,14 @@ class VirtualenvCreateError(ProjectError):
 class Backend(abc.ABC):
     """The base class for virtualenv backends"""
 
-    def __init__(self, project: Project, python: Optional[str]) -> None:
+    def __init__(self, project: Project, python: str | None) -> None:
         self.project = project
         self.python = python
 
+    @abc.abstractmethod
+    def pip_args(self, with_pip: bool) -> Iterable[str]:
+        pass
+
     @cached_property
     def _resolved_interpreter(self) -> PythonInfo:
         if not self.python:
@@ -52,9 +58,11 @@ class Backend(abc.ABC):
         """
         return self._resolved_interpreter.identifier
 
-    def subprocess_call(self, cmd: List[str], **kwargs: Any) -> None:
+    def subprocess_call(self, cmd: list[str], **kwargs: Any) -> None:
         self.project.core.ui.echo(
-            f"Run command: [green]{cmd}[/]", verbosity=termui.Verbosity.DETAIL, err=True
+            f"Run command: [success]{cmd}[/]",
+            verbosity=termui.Verbosity.DETAIL,
+            err=True,
         )
         try:
             subprocess.check_call(
@@ -70,13 +78,19 @@ class Backend(abc.ABC):
         if not location.exists():
             return
         if not force:
-            raise VirtualenvCreateError(f"The location {location} is not empty")
-        self.project.core.ui.echo(
-            f"Cleaning existing target directory {location}", err=True
-        )
-        shutil.rmtree(location)
+            raise VirtualenvCreateError(
+                f"The location {location} is not empty, add --force to overwrite it."
+            )
+        if location.is_file():
+            self.project.core.ui.echo(f"Removing existing file {location}", err=True)
+            location.unlink()
+        else:
+            self.project.core.ui.echo(
+                f"Cleaning existing target directory {location}", err=True
+            )
+            shutil.rmtree(location)
 
-    def get_location(self, name: Optional[str]) -> Path:
+    def get_location(self, name: str | None) -> Path:
         venv_parent = Path(self.project.config["venv.location"])
         if not venv_parent.is_dir():
             venv_parent.mkdir(exist_ok=True, parents=True)
@@ -84,49 +98,66 @@ class Backend(abc.ABC):
 
     def create(
         self,
-        name: Optional[str] = None,
-        args: Tuple[str, ...] = (),
+        name: str | None = None,
+        args: tuple[str, ...] = (),
         force: bool = False,
         in_project: bool = False,
+        prompt: str | None = None,
+        with_pip: bool = False,
     ) -> Path:
         if in_project:
             location = self.project.root / ".venv"
         else:
             location = self.get_location(name)
+        args = (*self.pip_args(with_pip), *args)
+        if prompt is not None:
+            prompt_string = prompt.format(
+                project_name=self.project.root.name.lower() or "virtualenv",
+                python_version=self.ident,
+            )
+            args = (*args, f"--prompt={prompt_string}")
         self._ensure_clean(location, force)
         self.perform_create(location, args)
         return location
 
     @abc.abstractmethod
-    def perform_create(self, location: Path, args: Tuple[str, ...]) -> None:
+    def perform_create(self, location: Path, args: tuple[str, ...]) -> None:
         pass
 
 
 class VirtualenvBackend(Backend):
-    def perform_create(self, location: Path, args: Tuple[str, ...]) -> None:
+    def pip_args(self, with_pip: bool) -> Iterable[str]:
+        if with_pip or self.project.config["venv.with_pip"]:
+            return ()
+        return ("--no-pip", "--no-setuptools", "--no-wheel")
+
+    def perform_create(self, location: Path, args: tuple[str, ...]) -> None:
         cmd = [
             sys.executable,
             "-m",
             "virtualenv",
-            "--no-pip",
-            "--no-setuptools",
-            "--no-wheel",
             str(location),
+            "-p",
+            str(self._resolved_interpreter.executable),
+            *args,
         ]
-        cmd.extend(["-p", str(self._resolved_interpreter.executable)])
-        cmd.extend(args)
         self.subprocess_call(cmd)
 
 
 class VenvBackend(VirtualenvBackend):
-    def perform_create(self, location: Path, args: Tuple[str, ...]) -> None:
+    def pip_args(self, with_pip: bool) -> Iterable[str]:
+        if with_pip or self.project.config["venv.with_pip"]:
+            return ()
+        return ("--without-pip",)
+
+    def perform_create(self, location: Path, args: tuple[str, ...]) -> None:
         cmd = [
             str(self._resolved_interpreter.executable),
             "-m",
             "venv",
-            "--without-pip",
             str(location),
-        ] + list(args)
+            *args,
+        ]
         self.subprocess_call(cmd)
 
 
@@ -139,7 +170,12 @@ class CondaBackend(Backend):
             return self.python
         return super().ident
 
-    def perform_create(self, location: Path, args: Tuple[str, ...]) -> None:
+    def pip_args(self, with_pip: bool) -> Iterable[str]:
+        if with_pip or self.project.config["venv.with_pip"]:
+            return ("pip",)
+        return ()
+
+    def perform_create(self, location: Path, args: tuple[str, ...]) -> None:
         if self.python:
             python_ver = self.python
         else:
@@ -160,7 +196,7 @@ class CondaBackend(Backend):
         self.subprocess_call(cmd)
 
 
-BACKENDS: Mapping[str, Type[Backend]] = {
+BACKENDS: Mapping[str, type[Backend]] = {
     "virtualenv": VirtualenvBackend,
     "venv": VenvBackend,
     "conda": CondaBackend,
