@@ -4,7 +4,6 @@ import argparse
 import json
 import os
 import sys
-from argparse import Action, _ArgumentGroup
 from collections import ChainMap, OrderedDict
 from concurrent.futures import ThreadPoolExecutor
 from json import dumps
@@ -26,11 +25,9 @@ from resolvelib.structs import DirectedGraph
 from rich.tree import Tree
 
 from pdm import termui
-from pdm.compat import importlib_metadata as im
 from pdm.exceptions import PdmArgumentError, PdmUsageError, ProjectError
 from pdm.formats import FORMATS
 from pdm.formats.base import make_array, make_inline_table
-from pdm.models.repositories import BaseRepository
 from pdm.models.requirements import (
     Requirement,
     filter_requirements_with_extras,
@@ -38,7 +35,6 @@ from pdm.models.requirements import (
     strip_extras,
 )
 from pdm.models.specifiers import get_specifier
-from pdm.project import Project
 from pdm.utils import (
     comparable_version,
     is_path_relative_to,
@@ -47,11 +43,16 @@ from pdm.utils import (
 )
 
 if TYPE_CHECKING:
+    from argparse import Action, _ArgumentGroup
+
     from packaging.version import Version
     from resolvelib.resolvers import RequirementInformation, ResolutionImpossible
 
     from pdm.compat import Distribution
+    from pdm.compat import importlib_metadata as im
     from pdm.models.candidates import Candidate
+    from pdm.models.repositories import BaseRepository
+    from pdm.project import Project
 
 
 class ErrorArgumentParser(argparse.ArgumentParser):
@@ -677,6 +678,7 @@ def merge_dictionary(
 ) -> None:
     """Merge the input dict with the target while preserving the existing values
     properly. This will update the target dictionary in place.
+    List values will be extended, but only if the value is not already in the list.
     """
     for key, value in input.items():
         if key not in target:
@@ -684,7 +686,7 @@ def merge_dictionary(
         elif isinstance(value, dict):
             target[key].update(value)
         elif isinstance(value, list):
-            target[key].extend(value)
+            target[key].extend([x for x in value if x not in target[key]])
         else:
             target[key] = value
 
@@ -693,6 +695,7 @@ def fetch_hashes(repository: BaseRepository, mapping: Mapping[str, Candidate]) -
     """Fetch hashes for candidates in parallel"""
 
     def do_fetch(candidate: Candidate) -> None:
+        termui.logger.info("Fetching hashes for %s", candidate)
         candidate.hashes = repository.get_hashes(candidate)
 
     with ThreadPoolExecutor() as executor:
@@ -727,7 +730,7 @@ def get_dist_location(dist: Distribution) -> str:
 
 
 def get_pep582_path(project: Project) -> str:
-    import importlib.resources
+    from pdm.compat import resources_open_binary
 
     script_dir = Path(__file__).parent.parent / "pep582"
     if script_dir.exists():
@@ -737,6 +740,6 @@ def get_pep582_path(project: Project) -> str:
     if script_dir.joinpath("sitecustomize.py").exists():
         return str(script_dir)
     script_dir.mkdir(parents=True, exist_ok=True)
-    with importlib.resources.open_binary("pdm.pep582", "sitecustomize.py") as f:
+    with resources_open_binary("pdm.pep582", "sitecustomize.py") as f:
         script_dir.joinpath("sitecustomize.py").write_bytes(f.read())
     return str(script_dir)
