@@ -5,7 +5,6 @@ import os
 import urllib.parse
 from pathlib import Path
 
-from pdm.exceptions import ProjectError
 from pdm.utils import expand_env_vars, path_to_url
 
 
@@ -124,10 +123,12 @@ class HatchBackend(BuildBackend):
 _BACKENDS: dict[str, type[BuildBackend]] = {
     "pdm-pep517": PDMLegacyBackend,
     "setuptools": SetuptoolsBackend,
-    "flit": FlitBackend,
+    "flit-core": FlitBackend,
     "hatchling": HatchBackend,
     "pdm-backend": PDMBackend,
 }
+# Fallback to the first backend
+DEFAULT_BACKEND = next(iter(_BACKENDS.values()))
 
 
 def get_backend(name: str) -> type[BuildBackend]:
@@ -140,12 +141,11 @@ def get_backend_by_spec(spec: dict) -> type[BuildBackend]:
     The parameter passed in is the 'build-system' section in pyproject.toml.
     """
     if "build-backend" not in spec:
-        # default to setuptools
-        return SetuptoolsBackend
+        return DEFAULT_BACKEND
     for backend_cls in _BACKENDS.values():
         if backend_cls.build_system()["build-backend"] == spec["build-backend"]:
             return backend_cls
-    raise ProjectError(f"Unsupported custom build backend: {spec['build-backend']}")
+    return DEFAULT_BACKEND
 
 
 def get_relative_path(url: str) -> str | None:
