@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import dataclasses
 import os
 import re
 import warnings
@@ -14,8 +15,10 @@ from unearth import Link, vcs_support
 
 from pdm import termui
 from pdm.builders import EditableBuilder, WheelBuilder
+from pdm.compat import cached_property
 from pdm.compat import importlib_metadata as im
 from pdm.exceptions import BuildError, CandidateNotFound, InvalidPyVersion
+from pdm.models.backends import get_backend, get_backend_by_spec
 from pdm.models.requirements import (
     FileRequirement,
     Requirement,
@@ -25,16 +28,14 @@ from pdm.models.requirements import (
 )
 from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet
-from pdm.project.metadata import MutableMetadata, SetupDistribution
+from pdm.project.project_file import PyProject
 from pdm.utils import (
-    cached_property,
+    cd,
     convert_hashes,
     create_tracked_tempdir,
-    expand_env_vars_in_auth,
     get_rev_from_url,
     get_venv_like_prefix,
     normalize_name,
-    path_replace,
     path_to_url,
     url_without_fragments,
 )
@@ -86,7 +87,9 @@ def _find_best_match_link(
     # the package sources.
     def attempt_to_find() -> Link | None:
         if hashes is None:
-            best = finder.find_best_match(req.as_line()).best
+            best = finder.find_best_match(
+                req.as_line(), allow_yanked=req.is_pinned
+            ).best
             return best.link if best is not None else None
         # We don't evaluate against the hashes, they will be validated later.
         evaluator = finder.build_evaluator(req.name)
@@ -178,8 +181,15 @@ class Candidate:
         return self._prepared.revision if self._prepared else "unknown"
 
     def __repr__(self) -> str:
-        source = getattr(self.link, "comes_from", "unknown")
-        return f"<Candidate {self.name} {self.version} from {source}>"
+        source = getattr(self.link, "comes_from", None)
+        from_source = f" from {source}" if source else ""
+        return f"<Candidate {self}{from_source}>"
+
+    def __str__(self) -> str:
+        if self.req.is_named:
+            return f"{self.name}@{self.version}"
+        assert self.link is not None
+        return f"{self.name}@{self.link.url_without_fragment}"
 
     @classmethod
     def from_installation_candidate(
@@ -218,13 +228,13 @@ class Candidate:
     @no_type_check
     def as_lockfile_entry(self, project_root: Path) -> dict[str, Any]:
         """Build a lockfile entry dictionary for the candidate."""
-        root_path = project_root.as_posix()
         result = {
             "name": normalize_name(self.name),
             "version": str(self.version),
             "extras": sorted(self.req.extras or ()),
             "requires_python": str(self.requires_python),
             "editable": self.req.editable,
+            "subdirectory": getattr(self.req, "subdirectory", None),
         }
         if self.req.is_vcs:
             result.update(
@@ -236,19 +246,16 @@ class Candidate:
             if not self.req.editable:
                 result.update(revision=self.get_revision())
         elif not self.req.is_named:
-            if self.req.is_file_or_url and self.req.is_local_dir:
-                result.update(path=path_replace(root_path, ".", self.req.str_path))
-            else:
-                result.update(
-                    url=path_replace(
-                        root_path.lstrip("/"), "${PROJECT_ROOT}", self.req.url
-                    )
-                )
+            with cd(project_root):
+                if self.req.is_file_or_url and self.req.is_local:
+                    result.update(path=self.req.str_path)
+                else:
+                    result.update(url=self.req.url)
         return {k: v for k, v in result.items() if v}
 
     def format(self) -> str:
         """Format for output."""
-        return f"[bold green]{self.name}[/] [yellow]{self.version}[/]"
+        return f"[req]{self.name}[/] [warning]{self.version}[/]"
 
     def prepare(self, environment: Environment) -> PreparedCandidate:
         """Prepare the candidate for installation."""
@@ -282,11 +289,8 @@ class PreparedCandidate:
     def _replace_url_vars(self, link: Link | None) -> Link | None:
         if link is None:
             return None
-        project_root = self.environment.project.root.as_posix()  # type: ignore
-        url = expand_env_vars_in_auth(link.normalized).replace(
-            "${PROJECT_ROOT}", project_root.lstrip("/")
-        )
-        return Link(url)
+        url = self.environment.project.backend.expand_line(link.normalized)
+        return dataclasses.replace(link, url=url)
 
     @cached_property
     def revision(self) -> str:
@@ -329,30 +333,23 @@ class PreparedCandidate:
                 }
             )
         elif isinstance(req, FileRequirement):
-            if req.is_local_dir:
+            assert self.link is not None
+            if self.link.is_file and self.link.file_path.is_dir():
                 return _filter_none(
                     {
-                        "url": url_without_fragments(req.url),
+                        "url": self.link.url_without_fragment,
                         "dir_info": _filter_none({"editable": req.editable or None}),
                         "subdirectory": req.subdirectory,
                     }
                 )
-            url = expand_env_vars_in_auth(
-                req.url.replace(
-                    "${PROJECT_ROOT}",
-                    self.environment.project.root.as_posix().lstrip(  # type: ignore
-                        "/"
-                    ),
-                )
-            )
             with self.environment.get_finder() as finder:
                 hash_cache = self.environment.project.make_hash_cache()
                 return _filter_none(
                     {
-                        "url": url_without_fragments(req.url),
+                        "url": self.link.url_without_fragment,
                         "archive_info": {
                             "hash": hash_cache.get_hash(
-                                Link(url), finder.session
+                                self.link, finder.session
                             ).replace(":", "=")
                         },
                         "subdirectory": req.subdirectory,
@@ -455,12 +452,12 @@ class PreparedCandidate:
         # Try getting from PEP 621 metadata
         pyproject_toml = self._unpacked_dir / "pyproject.toml"
         if pyproject_toml.exists():
-            try:
-                metadata = MutableMetadata.from_file(pyproject_toml)
-            except ValueError:
+            pyproject = PyProject(pyproject_toml, ui=self.environment.project.core.ui)
+            metadata = pyproject.metadata.unwrap()
+            if not metadata:
                 termui.logger.warn("Failed to parse pyproject.toml")
             else:
-                dynamic_fields = metadata.dynamic or []
+                dynamic_fields = metadata.get("dynamic", [])
                 # Use the parse result only when all are static
                 if set(dynamic_fields).isdisjoint(
                     {
@@ -471,15 +468,31 @@ class PreparedCandidate:
                         "requires-python",
                     }
                 ):
+                    try:
+                        backend_cls = get_backend_by_spec(pyproject.build_system)
+                    except Exception:
+                        # no variable expansion
+                        backend_cls = get_backend("setuptools")
+                    backend = backend_cls(self._unpacked_dir)
                     setup = Setup(
-                        name=metadata.name,
-                        summary=metadata.description,
-                        version=metadata.version,
-                        install_requires=metadata.dependencies or [],
-                        extras_require=metadata.optional_dependencies or {},
-                        python_requires=metadata.requires_python or None,
+                        name=metadata.get("name"),
+                        summary=metadata.get("description"),
+                        version=metadata.get("version"),
+                        install_requires=list(
+                            map(
+                                backend.expand_line,
+                                metadata.get("dependencies", []),
+                            )
+                        ),
+                        extras_require={
+                            k: list(map(backend.expand_line, v))
+                            for k, v in metadata.get(
+                                "optional-dependencies", {}
+                            ).items()
+                        },
+                        python_requires=metadata.get("requires-python"),
                     )
-                    return SetupDistribution(setup)
+                    return setup.as_dist()
         # If all fail, try building the source to get the metadata
         builder = EditableBuilder if self.req.editable else WheelBuilder
         try:
@@ -500,9 +513,9 @@ class PreparedCandidate:
                 termui.logger.warn(message)
                 warnings.warn(message, RuntimeWarning)
                 setup = Setup()
-            return SetupDistribution(setup)
+            return setup.as_dist()
         else:
-            return im.PathDistribution(Path(self._metadata_dir))
+            return im.PathDistribution(Path(cast(str, self._metadata_dir)))
 
     @property
     def metadata(self) -> im.Distribution:
