@@ -17,7 +17,6 @@ from packaging.requirements import InvalidRequirement
 from packaging.requirements import Requirement as PackageRequirement
 from packaging.specifiers import SpecifierSet
 from packaging.utils import parse_sdist_filename, parse_wheel_filename
-from packaging.version import parse as parse_version
 from unearth import Link
 
 from pdm.compat import Distribution
@@ -27,7 +26,9 @@ from pdm.models.markers import Marker, get_marker, split_marker_extras
 from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet, get_specifier
 from pdm.utils import (
+    PACKAGING_22,
     add_ssh_scheme_to_git_uri,
+    comparable_version,
     normalize_name,
     path_to_url,
     path_without_fragments,
@@ -36,6 +37,8 @@ from pdm.utils import (
 )
 
 if TYPE_CHECKING:
+    from typing import Match
+
     from pdm._types import RequirementDict
 
 
@@ -110,11 +113,7 @@ class Requirement:
         """Return a new requirement with the given pinned version."""
         if self.is_pinned or not other_version:
             return self
-        version = parse_version(other_version)
-        normalized = str(version)
-        if version.local:
-            # Remove the local part to accept wider range of prereleases.
-            normalized = normalized.rsplit("+", 1)[0]
+        normalized = comparable_version(other_version)
         return dataclasses.replace(self, specifier=get_specifier(f"=={normalized}"))
 
     def _hash_key(self) -> tuple:
@@ -267,7 +266,7 @@ class FileRequirement(Requirement):
             self._check_installable()
 
     def _hash_key(self) -> tuple:
-        return super()._hash_key() + (self.url, self.editable)
+        return super()._hash_key() + (self.get_full_url(), self.editable)
 
     @classmethod
     def create(cls: type[T], **kwargs: Any) -> T:
@@ -462,6 +461,34 @@ def filter_requirements_with_extras(
     return result
 
 
+_legacy_specifier_re = re.compile(r"(==|!=|<=|>=|<|>)(\s*)([^,;\s)]*)")
+
+
+def parse_as_pkg_requirement(line: str) -> PackageRequirement:
+    """Parse a requirement line as packaging.requirement.Requirement"""
+
+    def fix_wildcard(match: Match[str]) -> str:
+        operator, _, version = match.groups()
+        if ".*" not in version or operator in ("==", "!="):
+            return match.group(0)
+        version = version.replace(".*", ".0")
+        if operator in ("<", "<="):  # <4.* and <=4.* are equivalent to <4.0
+            operator = "<"
+        elif operator in (">", ">="):  # >4.* and >=4.* are equivalent to >=4.0
+            operator = ">="
+        return f"{operator}{version}"
+
+    try:
+        return PackageRequirement(line)
+    except InvalidRequirement:
+        if not PACKAGING_22:  # We can't do anything, reraise the error.
+            raise
+        # Since packaging 22.0, legacy specifiers like '>=4.*' are no longer
+        # supported. We try to normalize them to the new format.
+        new_line = _legacy_specifier_re.sub(fix_wildcard, line)
+        return PackageRequirement(new_line)
+
+
 def parse_requirement(line: str, editable: bool = False) -> Requirement:
 
     m = _vcs_req_re.match(line)
@@ -479,14 +506,14 @@ def parse_requirement(line: str, editable: bool = False) -> Requirement:
         if replaced:
             line = line.replace("{root:uri}", root_url)
         try:
-            package_req = PackageRequirement(line)  # type: ignore
+            pkg_req = parse_as_pkg_requirement(line)
         except InvalidRequirement as e:
             m = _file_req_re.match(line)
             if m is None:
                 raise RequirementError(str(e)) from None
             r = FileRequirement.create(**m.groupdict())
         else:
-            r = Requirement.from_pkg_requirement(package_req)
+            r = Requirement.from_pkg_requirement(pkg_req)
         if replaced:
             assert isinstance(r, FileRequirement)
             r.url = r.url.replace(root_url, "{root:uri}")
