@@ -1,23 +1,23 @@
 from __future__ import annotations
 
 import hashlib
-import json
 import os
 import re
 import shutil
 import sys
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Iterable, cast
-from urllib.parse import urlparse
 
 import platformdirs
 import tomlkit
 from findpython import Finder
 from tomlkit.items import Array
+from unearth import Link
 
 from pdm import termui
-from pdm._types import Source
+from pdm.compat import cached_property
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
+from pdm.models.backends import BuildBackend, get_backend_by_spec
 from pdm.models.caches import CandidateInfoCache, HashCache, WheelCache
 from pdm.models.candidates import Candidate, make_candidate
 from pdm.models.environment import Environment, GlobalEnvironment
@@ -26,10 +26,9 @@ from pdm.models.repositories import BaseRepository, LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import PySpecSet, get_specifier
 from pdm.project.config import Config
-from pdm.project.metadata import MutableMetadata as Metadata
+from pdm.project.lockfile import Lockfile
+from pdm.project.project_file import PyProject
 from pdm.utils import (
-    atomic_open_for_write,
-    cached_property,
     cd,
     deprecation_warning,
     expand_env_vars_in_auth,
@@ -42,8 +41,8 @@ from pdm.utils import (
 
 if TYPE_CHECKING:
     from resolvelib.reporters import BaseReporter
-    from rich.status import Status
 
+    from pdm._types import Source, Spinner
     from pdm.core import Core
     from pdm.resolver.providers import BaseProvider
 
@@ -62,8 +61,8 @@ class Project:
     """
 
     PYPROJECT_FILENAME = "pyproject.toml"
+    LOCKFILE_FILENAME = "pdm.lock"
     DEPENDENCIES_RE = re.compile(r"(?:(.+?)-)?dependencies")
-    LOCKFILE_VERSION = "4.0"
 
     def __init__(
         self,
@@ -72,8 +71,7 @@ class Project:
         is_global: bool = False,
         global_config: str | Path | None = None,
     ) -> None:
-        self._pyproject: dict | None = None
-        self._lockfile: dict | None = None
+        self._lockfile: Lockfile | None = None
         self._environment: Environment | None = None
         self._python: PythonInfo | None = None
         self.core = core
@@ -99,68 +97,31 @@ class Project:
             if self.global_config["global_project.fallback_verbose"]:
                 self.core.ui.echo(
                     "Project is not found, fallback to the global project",
-                    style="yellow",
+                    style="warning",
                     err=True,
                 )
 
-        self.root = Path(root_path or "").absolute()
+        self.root: Path = Path(root_path or "").absolute()
         self.is_global = is_global
         self.init_global_project()
-        self._lockfile_file = self.root / "pdm.lock"
 
     def __repr__(self) -> str:
         return f"<Project '{self.root.as_posix()}'>"
 
-    @property
-    def pyproject_file(self) -> Path:
-        return self.root / self.PYPROJECT_FILENAME
+    @cached_property
+    def pyproject(self) -> PyProject:
+        return PyProject(self.root / self.PYPROJECT_FILENAME, ui=self.core.ui)
 
     @property
-    def lockfile_file(self) -> Path:
-        return self._lockfile_file
-
-    @lockfile_file.setter
-    def lockfile_file(self, path: str) -> None:
-        self._lockfile_file = Path(path).absolute()
-        self._lockfile = None
-
-    @property
-    def pyproject(self) -> dict | None:
-        if not self._pyproject:
-            if self.pyproject_file.exists():
-                data = tomlkit.parse(self.pyproject_file.read_text("utf-8"))
-                self._pyproject = cast(dict, data)
-            else:
-                self._pyproject = cast(dict, tomlkit.document())
-        return self._pyproject
-
-    @pyproject.setter
-    def pyproject(self, data: dict[str, Any]) -> None:
-        self._pyproject = data
-
-    @property
-    def tool_settings(self) -> dict:
-        data = self.pyproject
-        if not data:
-            return {}
-        return data.setdefault("tool", {}).setdefault("pdm", {})
-
-    @property
-    def name(self) -> str | None:
-        return self.meta.get("name")
-
-    @property
-    def lockfile(self) -> dict:
-        if not self._lockfile:
-            if not self.lockfile_file.is_file():
-                raise ProjectError("Lock file does not exist.")
-            data = tomlkit.parse(self.lockfile_file.read_text("utf-8"))
-            self._lockfile = cast(dict, data)
+    def lockfile(self) -> Lockfile:
+        if self._lockfile is None:
+            self._lockfile = Lockfile(
+                self.root / self.LOCKFILE_FILENAME, ui=self.core.ui
+            )
         return self._lockfile
 
-    @lockfile.setter
-    def lockfile(self, data: dict[str, Any]) -> None:
-        self._lockfile = data
+    def set_lockfile(self, path: str | Path) -> None:
+        self._lockfile = Lockfile(path, ui=self.core.ui)
 
     @property
     def config(self) -> dict[str, Any]:
@@ -171,22 +132,25 @@ class Project:
 
     @property
     def scripts(self) -> dict[str, str | dict[str, str]]:
-        return self.tool_settings.get("scripts", {})  # type: ignore
+        return self.pyproject.settings.get("scripts", {})  # type: ignore
 
     @cached_property
     def project_config(self) -> Config:
         """Read-and-writable configuration dict for project settings"""
         return Config(self.root / ".pdm.toml")
 
+    @property
+    def name(self) -> str | None:
+        return self.pyproject.metadata.get("name")
+
     @property
     def python(self) -> PythonInfo:
         if not self._python:
             self._python = self.resolve_interpreter()
             if self._python.major < 3:
-                deprecation_warning(
-                    "Python 2.7 has reached EOL and PDM will remove the support "
-                    "in version 2.0. Please upgrade your Python to 3.6 or later.",
-                    raise_since="2.0",
+                raise PdmUsageError(
+                    "Python 2.7 has reached EOL and PDM no longer supports it. "
+                    "Please upgrade your Python to 3.6 or later.",
                 )
         return self._python
 
@@ -195,11 +159,6 @@ class Project:
         self._python = value
         self.project_config["python.path"] = value.path
 
-    @property
-    def python_executable(self) -> str:
-        """For backward compatibility"""
-        return str(self.python.executable)
-
     def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
         from pdm.cli.commands.venv.utils import get_venv_python, iter_venvs
@@ -209,7 +168,7 @@ class Project:
 
         def note(message: str) -> None:
             if not self.is_global:
-                self.core.ui.echo(message, style="yellow", err=True)
+                self.core.ui.echo(message, style="warning", err=True)
 
         config = self.config
         if config.get("python.path") and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
@@ -226,15 +185,15 @@ class Project:
                 python = PythonInfo.from_path(get_venv_python(Path(venv_in_env)))
                 if match_version(python):
                     note(
-                        f"Inside an active virtualenv [green]{venv_in_env}[/], "
-                        "reuse it."
+                        f"Inside an active virtualenv [success]{venv_in_env}[/], "
+                        "reusing it."
                     )
                     return python
             # otherwise, get a venv associated with the project
             for _, venv in iter_venvs(self):
                 python = PythonInfo.from_path(get_venv_python(venv))
                 if match_version(python):
-                    note(f"Virtualenv [green]{venv}[/] is reused.")
+                    note(f"Virtualenv [success]{venv}[/] is reused.")
                     self.python = python
                     return python
 
@@ -249,7 +208,10 @@ class Project:
 
         for py_version in self.find_interpreters():
             if match_version(py_version):
-                note("[green]__pypackages__[/] is detected, using the PEP 582 mode")
+                if config.get("python.use_venv"):
+                    note(
+                        "[success]__pypackages__[/] is detected, using the PEP 582 mode"
+                    )
                 self.python = py_version
                 return py_version
 
@@ -282,10 +244,12 @@ class Project:
         backend: str = self.config["venv.backend"]
         venv_backend = BACKENDS[backend](self, None)
         path = venv_backend.create(
-            force=True, in_project=self.config["venv.in_project"]
+            force=True,
+            in_project=self.config["venv.in_project"],
+            prompt=self.config["venv.prompt"],
         )
         self.core.ui.echo(
-            f"Virtualenv is created successfully at [green]{path}[/]", err=True
+            f"Virtualenv is created successfully at [success]{path}[/]", err=True
         )
         return path
 
@@ -301,13 +265,13 @@ class Project:
 
     @property
     def python_requires(self) -> PySpecSet:
-        return PySpecSet(self.meta.requires_python)
+        return PySpecSet(self.pyproject.metadata.get("requires-python", ""))
 
     def get_dependencies(self, group: str | None = None) -> dict[str, Requirement]:
-        metadata = self.meta
+        metadata = self.pyproject.metadata
         group = group or "default"
         optional_dependencies = metadata.get("optional-dependencies", {})
-        dev_dependencies = self.tool_settings.get("dev-dependencies", {})
+        dev_dependencies = self.pyproject.settings.get("dev-dependencies", {})
         in_metadata = group == "default" or group in optional_dependencies
         if group == "default":
             deps = metadata.get("dependencies", [])
@@ -317,7 +281,7 @@ class Project:
                     f"The {group} group exists in both [optional-dependencies] "
                     "and [dev-dependencies], the former is taken.",
                     err=True,
-                    style="yellow",
+                    style="warning",
                 )
             if group in optional_dependencies:
                 deps = optional_dependencies[group]
@@ -332,10 +296,10 @@ class Project:
                     if in_metadata:
                         self.core.ui.echo(
                             f"WARNING: Skipping editable dependency [b]{line}[/] in the"
-                            r" [green]\[project][/] table. Please move it to the "
-                            r"[green]\[tool.pdm.dev-dependencies][/] table",
+                            r" [success]\[project][/] table. Please move it to the "
+                            r"[success]\[tool.pdm.dev-dependencies][/] table",
                             err=True,
-                            style="yellow",
+                            style="warning",
                         )
                         continue
                     req = parse_requirement(line[3:].strip(), True)
@@ -352,7 +316,7 @@ class Project:
     @property
     def dev_dependencies(self) -> dict[str, Requirement]:
         """All development dependencies"""
-        dev_group = self.tool_settings.get("dev-dependencies", {})
+        dev_group = self.pyproject.settings.get("dev-dependencies", {})
         if not dev_group:
             return {}
         result = {}
@@ -368,10 +332,10 @@ class Project:
 
     def iter_groups(self) -> Iterable[str]:
         groups = {"default"}
-        if self.meta.optional_dependencies:
-            groups.update(self.meta.optional_dependencies.keys())
-        if self.tool_settings.get("dev-dependencies"):
-            groups.update(self.tool_settings["dev-dependencies"].keys())
+        if self.pyproject.metadata.get("optional-dependencies"):
+            groups.update(self.pyproject.metadata["optional-dependencies"].keys())
+        if self.pyproject.settings.get("dev-dependencies"):
+            groups.update(self.pyproject.settings["dev-dependencies"].keys())
         return groups
 
     @property
@@ -380,49 +344,55 @@ class Project:
 
     @property
     def allow_prereleases(self) -> bool | None:
-        return self.tool_settings.get("allow_prereleases")
+        return self.pyproject.settings.get("allow_prereleases")
 
     @property
     def default_source(self) -> Source:
-        """Get the default source of from the pypi setting"""
+        """Get the default source from the pypi setting"""
         return cast(
             "Source",
             {
                 "url": self.config["pypi.url"],
                 "verify_ssl": self.config["pypi.verify_ssl"],
                 "name": "pypi",
+                "username": self.config.get("pypi.username"),
+                "password": self.config.get("pypi.password"),
             },
         )
 
     @property
     def sources(self) -> list[Source]:
-        sources = list(self.tool_settings.get("source", []))
-        if all(source.get("name") != "pypi" for source in sources):
-            sources.insert(0, self.default_source)
-        expanded_sources: list[Source] = [
-            Source(
-                url=expand_env_vars_in_auth(s["url"]),
-                verify_ssl=s.get("verify_ssl", True),
-                name=s.get("name", urlparse(s["url"]).hostname),
-                type=s.get("type", "index"),
+        sources = list(self.pyproject.settings.get("source", []))
+        if not self.config.get("pypi.ignore_stored_index", False):
+            if all(source.get("name") != "pypi" for source in sources):
+                sources.insert(0, self.default_source)
+            stored_sources = dict(self.project_config.iter_sources())
+            stored_sources.update(
+                (k, v)
+                for k, v in self.global_config.iter_sources()
+                if k not in stored_sources
             )
-            for s in sources
+            # The order is kept as project sources -> global sources
+            sources.extend(stored_sources.values())
+        expanded_sources = [
+            cast("Source", {**source, "url": expand_env_vars_in_auth(source["url"])})
+            for source in sources
         ]
         return expanded_sources
 
-    def get_repository(self, cls: type[BaseRepository] | None = None) -> BaseRepository:
+    def get_repository(
+        self, cls: type[BaseRepository] | None = None, ignore_compatibility: bool = True
+    ) -> BaseRepository:
         """Get the repository object"""
         if cls is None:
             cls = self.core.repository_class
         sources = self.sources or []
-        return cls(sources, self.environment)
+        return cls(sources, self.environment, ignore_compatibility=ignore_compatibility)
 
     @property
     def locked_repository(self) -> LockedRepository:
-        import copy
-
         try:
-            lockfile = copy.deepcopy(self.lockfile)
+            lockfile = self.lockfile._data.unwrap()
         except ProjectError:
             lockfile = {}
 
@@ -433,6 +403,7 @@ class Project:
         strategy: str = "all",
         tracked_names: Iterable[str] | None = None,
         for_install: bool = False,
+        ignore_compatibility: bool = True,
     ) -> BaseProvider:
         """Build a provider class for resolver.
 
@@ -448,11 +419,10 @@ class Project:
             ReusePinProvider,
         )
 
-        repository = self.get_repository()
+        repository = self.get_repository(ignore_compatibility=ignore_compatibility)
         allow_prereleases = self.allow_prereleases
         overrides = {
-            normalize_name(k): v
-            for k, v in self.tool_settings.get("overrides", {}).items()
+            normalize_name(k): v for k, v in self.pyproject.resolution_overrides.items()
         }
         locked_repository: LockedRepository | None = None
         if strategy != "all" or for_install:
@@ -463,7 +433,7 @@ class Project:
                     raise
                 self.core.ui.echo(
                     "Unable to reuse the lock file as it is not compatible with PDM",
-                    style="yellow",
+                    style="warning",
                     err=True,
                 )
 
@@ -487,7 +457,7 @@ class Project:
         self,
         requirements: list[Requirement],
         tracked_names: Iterable[str] | None = None,
-        spinner: Status | termui.DummySpinner | None = None,
+        spinner: Spinner | None = None,
     ) -> BaseReporter:
         """Return the reporter object to construct a resolver.
 
@@ -498,85 +468,85 @@ class Project:
         """
         from pdm.resolver.reporters import SpinnerReporter
 
-        return SpinnerReporter(spinner or termui.DummySpinner(), requirements)
+        if spinner is None:
+            spinner = termui.SilentSpinner("")
+
+        return SpinnerReporter(spinner, requirements)
 
     def get_lock_metadata(self) -> dict[str, Any]:
-        content_hash = tomlkit.string("sha256:" + self.get_content_hash("sha256"))
+        content_hash = tomlkit.string("sha256:" + self.pyproject.content_hash("sha256"))
         content_hash.trivia.trail = "\n\n"
-        return {"lock_version": self.LOCKFILE_VERSION, "content_hash": content_hash}
+        return {
+            "lock_version": self.lockfile.spec_version,
+            "content_hash": content_hash,
+        }
 
     def write_lockfile(
         self, toml_data: dict, show_message: bool = True, write: bool = True
     ) -> None:
         toml_data["metadata"].update(self.get_lock_metadata())
+        self.lockfile.set_data(toml_data)
 
         if write:
-            with atomic_open_for_write(self.lockfile_file) as fp:
-                tomlkit.dump(toml_data, fp)  # type: ignore
-            if show_message:
-                self.core.ui.echo("Changes are written to [green]pdm.lock[/].")
-            self._lockfile = None
-        else:
-            self._lockfile = toml_data
+            self.lockfile.write(show_message)
 
     def make_self_candidate(self, editable: bool = True) -> Candidate:
         req = parse_requirement(path_to_url(self.root.as_posix()), editable)
-        req.name = self.meta.name
-        return make_candidate(req, name=self.meta.name, version=self.meta.version)
-
-    def get_content_hash(self, algo: str = "md5") -> str:
-        # Only calculate sources and dependencies groups. Otherwise lock file is
-        # considered as unchanged.
-        dump_data = {
-            "sources": self.tool_settings.get("source", []),
-            "dependencies": self.meta.get("dependencies", []),
-            "dev-dependencies": self.tool_settings.get("dev-dependencies", {}),
-            "optional-dependencies": self.meta.get("optional-dependencies", {}),
-            "requires-python": self.meta.get("requires-python", ""),
-            "overrides": self.tool_settings.get("overrides", {}),
-        }
-        pyproject_content = json.dumps(dump_data, sort_keys=True)
-        hasher = hashlib.new(algo)
-        hasher.update(pyproject_content.encode("utf-8"))
-        return hasher.hexdigest()
+        assert self.name
+        req.name = self.name
+        can = make_candidate(req, name=self.name, link=Link.from_path(self.root))
+        can.prepare(self.environment).metadata
+        return can
 
     def is_lockfile_hash_match(self) -> bool:
-        if not self.lockfile_file.exists():
-            return False
-        hash_in_lockfile = str(
-            self.lockfile.get("metadata", {}).get("content_hash", "")
-        )
+        hash_in_lockfile = str(self.lockfile.hash)
         if not hash_in_lockfile:
             return False
         algo, hash_value = hash_in_lockfile.split(":")
-        content_hash = self.get_content_hash(algo)
+        content_hash = self.pyproject.content_hash(algo)
         return content_hash == hash_value
 
     def is_lockfile_compatible(self) -> bool:
-        if not self.lockfile_file.exists():
+        """Within the same major version, the higher lockfile generator can work with
+        lower lockfile but not vice versa.
+        """
+        if not self.lockfile.exists:
             return True
-        lockfile_version = str(
-            self.lockfile.get("metadata", {}).get("lock_version", "")
-        )
+        lockfile_version = str(self.lockfile.file_version)
         if not lockfile_version:
             return False
         if "." not in lockfile_version:
             lockfile_version += ".0"
-        accepted = get_specifier(f"~={lockfile_version}")
-        return accepted.contains(self.LOCKFILE_VERSION)
+        accepted = get_specifier(f"~={lockfile_version},>={lockfile_version}")
+        return accepted.contains(self.lockfile.spec_version)
 
-    def get_pyproject_dependencies(self, group: str, dev: bool = False) -> list[str]:
-        """Get the dependencies array in the pyproject.toml"""
+    def get_pyproject_dependencies(
+        self, group: str, dev: bool = False
+    ) -> tuple[list[str], bool]:
+        """Get the dependencies array in the pyproject.toml
+        Return a tuple of two elements, the first is the dependencies array,
+        and the second tells whether it is a dev-dependencies group.
+        """
+        metadata, settings = self.pyproject.metadata, self.pyproject.settings
         if group == "default":
-            return self.meta.setdefault("dependencies", [])
+            return metadata.setdefault("dependencies", []), False
         deps_dict = {
-            False: self.meta.setdefault("optional-dependencies", {}),
-            True: self.tool_settings.setdefault("dev-dependencies", {}),
+            False: metadata.get("optional-dependencies", {}),
+            True: settings.get("dev-dependencies", {}),
         }
-        for deps in deps_dict.values():
+        for is_dev, deps in deps_dict.items():
             if group in deps:
-                return deps[group]
-        return deps_dict[dev].setdefault(group, [])
+                return deps[group], is_dev
+        if dev:
+            return (
+                settings.setdefault("dev-dependencies", {}).setdefault(group, []),
+                dev,
+            )
+        else:
+            return (
+                metadata.setdefault("optional-dependencies", {}).setdefault(group, []),
+                dev,
+            )
 
     def add_dependencies(
         self,
@@ -585,47 +555,32 @@ class Project:
         dev: bool = False,
         show_message: bool = True,
     ) -> None:
-        deps = cast(Array, self.get_pyproject_dependencies(to_group, dev))
-        deps.multiline(True)
+        deps, is_dev = self.get_pyproject_dependencies(to_group, dev)
+        cast(Array, deps).multiline(True)
         for _, dep in requirements.items():
             matched_index = next(
                 (i for i, r in enumerate(deps) if dep.matches(r)),
                 None,
             )
+            req = dep.as_line()
             if matched_index is None:
-                deps.append(dep.as_line())
+                deps.append(req)
             else:
-                req = dep.as_line()
                 deps[matched_index] = req
-        self.write_pyproject(show_message)
-
-    def write_pyproject(self, show_message: bool = True) -> None:
-        with atomic_open_for_write(
-            self.pyproject_file.as_posix(), encoding="utf-8"
-        ) as f:
-            tomlkit.dump(self.pyproject, f)  # type: ignore
-        if show_message:
-            self.core.ui.echo("Changes are written to [green]pyproject.toml[/].")
-        self._pyproject = None
-
-    @property
-    def meta(self) -> Metadata:
-        if not self.pyproject:
-            self.pyproject = {"project": tomlkit.table()}
-        return Metadata(self.root, self.pyproject)
+        self.pyproject.write(show_message)
 
     def init_global_project(self) -> None:
-        if not self.is_global:
+        if not self.is_global or not self.pyproject.empty:
             return
-        if not self.pyproject_file.exists():
-            self.root.mkdir(parents=True, exist_ok=True)
-            self.pyproject_file.write_text(
-                """\
-[project]
-dependencies = ["pip", "setuptools", "wheel"]
-"""
-            )
-            self._pyproject = None
+        self.root.mkdir(parents=True, exist_ok=True)
+        self.pyproject.set_data(
+            {"project": {"dependencies": ["pip", "setuptools", "wheel"]}}
+        )
+        self.pyproject.write()
+
+    @property
+    def backend(self) -> BuildBackend:
+        return get_backend_by_spec(self.pyproject.build_system)(self.root)
 
     @property
     def cache_dir(self) -> Path:
@@ -679,11 +634,12 @@ dependencies = ["pip", "setuptools", "wheel"]
             args = []
         else:
             if not all(c.isdigit() for c in python_spec.split(".")):
-                if Path(python_spec).exists():
+                path = Path(python_spec)
+                if path.exists():
                     python = find_python_in_path(python_spec)
                     if python:
                         yield PythonInfo.from_path(python)
-                else:
+                if len(path.parts) == 1:  # only check for spec with only one part
                     python = shutil.which(python_spec)
                     if python:
                         yield PythonInfo.from_path(python)
@@ -704,3 +660,19 @@ dependencies = ["pip", "setuptools", "wheel"]
         if self.config["python.use_venv"]:
             finder._providers.insert(0, VenvProvider(self))
         return finder
+
+    # compatibility, shouldn't be used directly
+    @property
+    def meta(self) -> dict[str, Any]:
+        deprecation_warning(
+            "project.meta is deprecated, use project.pyproject.metadata instead"
+        )
+        return self.pyproject.metadata
+
+    @property
+    def tool_settings(self) -> dict[str, Any]:
+        deprecation_warning(
+            "project.tool_settings is deprecated, "
+            "use project.pyproject.settings instead"
+        )
+        return self.pyproject.settings
