@@ -31,7 +31,7 @@ class BaseProvider(AbstractProvider):
         self,
         repository: BaseRepository,
         allow_prereleases: bool | None = None,
-        overrides: dict[str, str] = None,
+        overrides: dict[str, str] | None = None,
     ) -> None:
         self.repository = repository
         self.allow_prereleases = allow_prereleases  # Root allow_prereleases value
@@ -165,9 +165,9 @@ class BaseProvider(AbstractProvider):
         if not requirement.is_named:
             backend = self.repository.environment.project.backend
             return not candidate.req.is_named and backend.expand_line(
-                url_without_fragments(candidate.req.url)  # type: ignore
+                url_without_fragments(candidate.req.get_full_url())  # type: ignore
             ) == backend.expand_line(
-                url_without_fragments(requirement.url)  # type: ignore
+                url_without_fragments(requirement.get_full_url())  # type: ignore
             )
         version = candidate.version
         this_name = self.repository.environment.project.name
@@ -211,7 +211,10 @@ class BaseProvider(AbstractProvider):
         new_requires_python = (
             candidate.req.requires_python & self.repository.environment.python_requires
         )
-        if not requires_python.is_superset(new_requires_python):
+        if (
+            candidate.identify() not in self.overrides
+            and not requires_python.is_superset(new_requires_python)
+        ):
             valid_deps.append(PythonRequirement.from_pyspec_set(requires_python))
         return valid_deps
 
