@@ -23,10 +23,17 @@ from typing import IO, Any, Iterator
 from packaging.version import Version
 
 from pdm._types import Source
-from pdm.compat import Distribution
+from pdm.compat import Distribution, importlib_metadata
 
 _egg_fragment_re = re.compile(r"(.*)[#&]egg=[^&]*")
 
+try:
+    _packaging_version = importlib_metadata.version("packaging")
+except Exception:
+    from packaging import __version__ as _packaging_version
+
+PACKAGING_22 = Version(_packaging_version) >= Version("22")
+
 
 def create_tracked_tempdir(
     suffix: str | None = None, prefix: str | None = None, dir: str | None = None
@@ -429,3 +436,8 @@ def path_without_fragments(path: str) -> Path:
     if not match:
         return Path(path)
     return Path(match.group(1))
+
+
+def is_in_zipapp() -> bool:
+    """Check if the current process is running in a zipapp"""
+    return not os.path.exists(__file__)
